import { h, Host } from '@stencil/core';
import Dynamsoft from "dwt";
import { download, edit, exitFullscreen, fitWindow, fullscreen, openFile, origSize, scanner, settings, sidebar } from './assets/base64';
export class PDFViewer {
  constructor() {
    this.containerID = "dwtcontrolContainer";
    this.thumbnailShown = true;
    this.status = "";
    this.percent = 100;
    this.showFitWindow = true;
    this.showActionOverlay = false;
    this.fullscreen = false;
    this.totalPageNumber = 0;
    this.selectedPageNumber = 0;
    this.url = undefined;
    this.license = undefined;
    this.showthumbnailviewer = undefined;
    this.uselocalservice = undefined;
  }
  componentWillLoad() {
    this.status = "Loading...";
  }
  componentDidLoad() {
    document.addEventListener("fullscreenchange", () => {
      if (document.fullscreenElement) {
        this.fullscreen = true;
      }
      else {
        this.fullscreen = false;
      }
    });
    this.initDWT();
  }
  initDWT() {
    Dynamsoft.DWT.ResourcesPath = "https://unpkg.com/dwt@18.0.0/dist";
    if (this.uselocalservice === "true") {
      Dynamsoft.DWT.UseLocalService = true;
    }
    else {
      Dynamsoft.DWT.UseLocalService = false;
    }
    let pThis = this;
    Dynamsoft.DWT.RegisterEvent('OnWebTwainReady', () => {
      Dynamsoft.DWT.CreateDWTObjectEx({
        WebTwainId: 'dwtcontrol'
      }, function (obj) {
        pThis.status = "";
        pThis.DWObject = obj;
        pThis.DWObject.Viewer.bind(pThis.dwtContainer);
        pThis.DWObject.Viewer.show();
        pThis.DWObject.Viewer.width = "100%";
        pThis.DWObject.Viewer.height = "100%";
        if (pThis.webTWAINReady) {
          pThis.webTWAINReady.emit(pThis.DWObject);
        }
        pThis.DWObject.Viewer.setViewMode(1, 1);
        pThis.DWObject.Viewer.cursor = "pointer";
        pThis.DWObject.RegisterEvent('OnBufferChanged', function (bufferChangeInfo) {
          if (bufferChangeInfo.action === "add" || bufferChangeInfo.action === "remove") {
            pThis.updateTotalPage();
          }
          pThis.updateSelectedPageNumber(pThis.DWObject.CurrentImageIndexInBuffer + 1);
        });
        pThis.thumbnailViewer = pThis.DWObject.Viewer.createThumbnailViewer();
        if (pThis.showthumbnailviewer === "true") {
          pThis.thumbnailViewer.show();
          pThis.thumbnailShown = true;
        }
        else {
          pThis.thumbnailShown = false;
        }
        pThis.loadPDF();
      }, function (err) {
        console.log(err);
      });
    });
    Dynamsoft.DWT.Containers = [{
        WebTwainId: 'dwtObject'
      }];
    if (this.license) {
      Dynamsoft.DWT.ProductKey = this.license;
    }
    Dynamsoft.DWT.Load();
  }
  async loadPDF() {
    if (this.url) {
      this.status = "Loading PDF...";
      try {
        let response = await fetch(this.url);
        let blob = await response.blob();
        let pThis = this;
        this.DWObject.LoadImageFromBinary(blob, function () {
          pThis.DWObject.Viewer.singlePageMode = true;
          pThis.DWObject.SelectImages([0]);
        }, function () { });
        this.status = "";
      }
      catch (error) {
        this.status = "Failed to load the PDF";
      }
    }
  }
  async toggleThumbnailViewer() {
    if (this.thumbnailShown) {
      this.thumbnailViewer.hide();
    }
    else {
      this.thumbnailViewer.show();
    }
    this.thumbnailShown = !this.thumbnailShown;
  }
  updateZoom(e) {
    this.percent = e.target.value;
    const zoom = this.percent / 100;
    this.DWObject.Viewer.zoom = zoom;
  }
  quicksize() {
    if (this.showFitWindow) {
      this.DWObject.Viewer.fitWindow("width");
      this.percent = this.DWObject.Viewer.zoom * 100;
    }
    else {
      this.DWObject.Viewer.zoom = 1.0;
      this.percent = 100;
    }
    this.showFitWindow = !this.showFitWindow;
  }
  renderStatus() {
    if (this.status) {
      return (h("div", { class: "status" }, this.status));
    }
  }
  renderActionOverlay() {
    let className;
    if (this.showActionOverlay) {
      className = "overlay";
    }
    else {
      className = "overlay hidden";
    }
    return (h("div", { class: className }, this.fullscreen
      ? h("img", { title: "Exit fullscreen", class: "Icon", src: exitFullscreen, onClick: () => this.toggleFullscreen() })
      : h("img", { title: "Enter fullscreen", class: "Icon", src: fullscreen, onClick: () => this.toggleFullscreen() }), h("img", { title: "Edit", class: "Icon", src: edit, onClick: () => this.edit() }), h("img", { title: "Scan documents", class: "Icon", src: scanner, onClick: () => this.scan() }), h("img", { title: "Load local file", class: "Icon", src: openFile, onClick: () => this.loadFile() }), h("img", { title: "Save to PDF", class: "Icon", src: download, onClick: () => this.saveFile() })));
  }
  async toggleFullscreen() {
    let isSafari = Dynamsoft.Lib.env.bSafari;
    if (isSafari) {
      if (this.parentContainer.classList.contains("fullscreen")) {
        this.parentContainer.classList.remove("fullscreen");
        this.fullscreen = false;
        this.resizeViewer();
      }
      else {
        this.parentContainer.classList.add("fullscreen");
        this.fullscreen = true;
        this.resizeViewer();
      }
    }
    else {
      if (document.fullscreenElement) {
        document.exitFullscreen();
      }
      else {
        let ele = this.parentContainer.parentNode["host"];
        await ele.requestFullscreen();
      }
    }
  }
  resizeViewer() {
    this.DWObject.Viewer.singlePageMode = true;
  }
  loadFile() {
    this.DWObject.LoadImageEx("", Dynamsoft.DWT.EnumDWT_ImageType.IT_ALL);
  }
  edit() {
    const imageEditor = this.DWObject.Viewer.createImageEditor();
    imageEditor.show();
  }
  scan() {
    let pThis = this;
    if (Dynamsoft.Lib.env.bMobile || this.uselocalservice != "true") {
      pThis.DWObject.Addon.Camera.scanDocument();
    }
    else {
      pThis.DWObject.SelectSource(function () {
        pThis.DWObject.OpenSource();
        pThis.DWObject.AcquireImage();
      }, function () {
        console.log("SelectSource failed!");
      });
    }
  }
  saveFile() {
    this.DWObject.SaveAllAsPDF("scanned.pdf");
  }
  toggleActionOverlay() {
    this.showActionOverlay = !this.showActionOverlay;
  }
  updateTotalPage() {
    if (this.DWObject) {
      this.totalPageNumber = this.DWObject.HowManyImagesInBuffer;
    }
  }
  updateSelectedPageNumber(num) {
    if (num <= 0 || num > this.totalPageNumber) {
      this.selectedPageNumber = this.DWObject.CurrentImageIndexInBuffer + 1;
      return;
    }
    this.selectedPageNumber = num;
    let index = this.selectedPageNumber - 1;
    if (this.DWObject) {
      this.DWObject.SelectImages([index]);
    }
  }
  render() {
    //const sideBar = getAssetPath(`./assets/sidebar.svg`);
    //const fitWindow = getAssetPath(`./assets/FitWindow.png`);
    //const originalSize = getAssetPath(`./assets/Orig_size.png`);
    return (h(Host, null, h("div", { class: "container", ref: (el) => this.parentContainer = el }, h("div", { class: "toolbar", ref: (el) => this.toolbar = el }, h("div", { class: "toolbar-item" }, h("img", { title: "Toggle thumbnail viewer", class: "Icon", src: sidebar, onClick: () => this.toggleThumbnailViewer() })), h("div", { class: "zoom toolbar-item" }, h("input", { type: "number", id: "percent-input", value: this.percent, title: "Percent", onChange: (e) => this.updateZoom(e) }), h("label", { htmlFor: "percent-input" }, "%")), h("div", { class: "quicksize toolbar-item" }, this.showFitWindow
      ? h("img", { title: "Fit window", class: "Icon", src: fitWindow, onClick: () => this.quicksize() })
      : h("img", { title: "Original size", class: "Icon", src: origSize, onClick: () => this.quicksize() })), h("div", { class: "page toolbar-item" }, h("input", { type: "number", id: "page-input", value: this.selectedPageNumber, title: "Page number", onChange: (e) => this.updateSelectedPageNumber(e.target.value) }), "/", this.totalPageNumber), h("div", { class: "toolbar-container toolbar-item" }), h("div", { class: "action toolbar-item" }, h("img", { class: "Icon", src: settings, onClick: () => this.toggleActionOverlay() }))), h("div", { class: "viewer-container" }, h("div", { id: this.containerID, class: "dwt-container", ref: (el) => this.dwtContainer = el }, this.renderStatus(), this.renderActionOverlay()))), h("slot", null)));
  }
  static get is() { return "pdf-viewer"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["pdf-viewer.css"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["pdf-viewer.css"]
    };
  }
  static get assetsDirs() { return ["assets"]; }
  static get properties() {
    return {
      "url": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "url",
        "reflect": false
      },
      "license": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "license",
        "reflect": false
      },
      "showthumbnailviewer": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "showthumbnailviewer",
        "reflect": false
      },
      "uselocalservice": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "uselocalservice",
        "reflect": false
      }
    };
  }
  static get states() {
    return {
      "status": {},
      "percent": {},
      "showFitWindow": {},
      "showActionOverlay": {},
      "fullscreen": {},
      "totalPageNumber": {},
      "selectedPageNumber": {}
    };
  }
  static get events() {
    return [{
        "method": "webTWAINReady",
        "name": "webTWAINReady",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "WebTwain",
          "resolved": "WebTwain",
          "references": {
            "WebTwain": {
              "location": "import",
              "path": "dwt/dist/types/WebTwain"
            }
          }
        }
      }];
  }
  static get methods() {
    return {
      "toggleThumbnailViewer": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
}
//# sourceMappingURL=pdf-viewer.js.map
